#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "fileoutput.h"

#define MAX_COLORS 256
#define START_SIZE 30

/*
   Stereogram - Copyright 2005 by Michael Kohn
   http://www.mikekohn.net/  mike@mikekohn.net

   This code is NOT opensource and you may not use any of it
   in your own projects.

*/

unsigned char colors0[256],colors1[256];

struct _bitmap_file bmp_file_in;

int add_color(int *color_que, int *que_ptr, int *que_len)
{
int t;

  for (t=*que_len; t>*que_ptr; t--)
  {
    color_que[t]=color_que[t-1];
  }

  color_que[*que_ptr]=rand()%MAX_COLORS;
  if (color_que[*que_ptr]<0) color_que[*que_ptr]=-color_que[*que_ptr];

  *que_len=(*que_len)+1;

  return 0;
}

int sub_color(int *color_que, int *que_ptr, int *que_len)
{
int t;

  for (t=*que_ptr; t<*que_len; t++)
  {
    color_que[t]=color_que[t+1];
  }

  *que_len=(*que_len)-1;
  if (*que_ptr>=*que_len) *que_ptr=0;

  return 0;
}

int main(int argc, char *argv[])
{
FILE *in,*out;
char outname[1024],inname[1024];
struct _pic_info pic_info;
int x,y,t;
int colors[MAX_COLORS];
int color_que[MAX_COLORS];
int que_len,que_ptr;
int color_stack[MAX_COLORS];
int stack_ptr;
int start_size,delta;

  printf("\nStereogram generator - Copyright 2005 Michael Kohn\n"); 
  printf("Date Version: November 2, 2005\n\n");

  if (argc<3)
  {
    printf("Usage: stereogram [ -options ] <image.bmp> <output image.bmp>\n");
    printf("    -width <pattern width (default 30) >\n");
    printf("    -delta <num pixel change (default 3) >\n\n");
    exit(0);
  }

  start_size=START_SIZE;
  delta=3;

  x=0;
  for (t=1; t<argc; t++)
  {
    if (strcmp(argv[t],"-width")==0)
    {
      start_size=atoi(argv[++t]);
    }
      else
    if (strcmp(argv[t],"-delta")==0)
    {
      delta=atoi(argv[++t]);
    }
      else
    if (x==0)
    {
      strcpy(inname,argv[t]);
      x++;
    }
      else
    if (x==1)
    {
      strcpy(outname,argv[t]);
    }
  }

  in=fopen(inname,"rb");
  if (in==0)
  {
    printf("Could not open input file: %s\n",inname);
    exit(0);
  }

  out=fopen(outname,"wb");
  if (out==0)
  {
    printf("Could not open output file: %s\n",outname);
    fclose(in);
    exit(0);
  }

  pic_info.picture=0;

  parse_bmp(in,&pic_info);

  srand(time(NULL));
  for (x=0; x<MAX_COLORS; x++)
  {
    colors[x]=rand()%0xffffff;
    if (colors[x]<0) colors[x]=-colors[x];
  }

  write_bmp_header(out,pic_info.width,pic_info.height,colors);

  for (y=pic_info.height-1; y>=0; y--)
  {
    que_len=start_size;

    for (x=0; x<que_len; x++)
    {
      color_que[x]=rand()%MAX_COLORS;
      if (color_que[x]<0) color_que[x]=-color_que[x];
    }

    que_ptr=0;
    stack_ptr=0;

    color_stack[stack_ptr++]=pic_info.picture[y*pic_info.width];

    for (x=0; x<pic_info.width; x++)
    {
      if (color_stack[stack_ptr-1]!=pic_info.picture[x+(y*pic_info.width)])
      {
        if (stack_ptr==1 && stack_ptr<MAX_COLORS)
        {
          color_stack[stack_ptr++]=pic_info.picture[x+(y*pic_info.width)];
          for (t=0; t<delta; t++)
          { add_color(color_que,&que_ptr,&que_len); }
        }
          else
        if (color_stack[stack_ptr-2]!=pic_info.picture[x+(y*pic_info.width)])
        {
          color_stack[stack_ptr++]=pic_info.picture[x+(y*pic_info.width)];
          for (t=0; t<delta; t++)
          { add_color(color_que,&que_ptr,&que_len); }
        }
          else
        if (color_stack[stack_ptr-2]==pic_info.picture[x+(y*pic_info.width)])
        {
          stack_ptr--;
          for (t=0; t<delta; t++)
          { sub_color(color_que,&que_ptr,&que_len); }
        }
      }

/*
      if (y>300 && y<400 && x==400) add_color(color_que,&que_ptr,&que_len);
      if (y>300 && y<400 && x==600) sub_color(color_que,&que_ptr,&que_len);
*/

      /* write_bmp_pixel(out,colors[color_que[que_ptr]]); */
      write_bmp_pixel(out,color_que[que_ptr]);

      que_ptr++;
      if (que_ptr>=que_len) que_ptr=0;
    }
  }

  write_bmp_footer(out);
 
  if (pic_info.picture!=0) free(pic_info.picture);

/*
  read_bitmap_file(in,&bmp_file_in);
  read_bitmap_file(in1,&bmp_file_in1);

  read_bitmap_info(in,&bmp_info_in);
  read_bitmap_info(in1,&bmp_info_in1);

  if (bmp_info_in.biCompression==2 || bmp_info_in.biCompression==1)
  {
    printf("RLE8 and RLE4 compression is not supported at this time.\n");
  }
    else
  {
#ifdef DEBUG
    display_bmp_info(0, &bmp_file_in, &bmp_info_in);
#endif

    for (t=0; t<bmp_info_in.biClrImportant; t++)
    {
      c=(getc(in)+getc(in)+getc(in))/3;
      getc(in);
      colors[t]=c;
    }

    for (t=0; t<bmp_info_in1.biClrImportant; t++)
    {
      c=(getc(in1)+getc(in1)+getc(in1))/3;
      getc(in1);
      colors1[t]=c;
    }

    fseek(in,bmp_file_in.bfOffs,0);
    write_bmp_header(out,bmp_info_in.biWidth,bmp_info_in.biHeight);

    if (bmp_info_in.biCompression==0)
    {
      raw_uncompressed(out,in,in1,&bmp_info_in);
    }
      else
    if (bmp_info_in.biCompression==1)
    {
    }
      else
    {
      printf("This type of compression is not supported at this time.\n");
    }
  }
*/

  fclose(in);
  fclose(out);

  printf("\n");

  return 0;
}

